function y = BMIV(K,S0,r,q,T,alpha,L,sigma)

% Brigo and Mercurio implied volatility from lognormal mixtures
% K = strike
% S0 = spot
% r,q = risk free rate, dividend yield
% T = maturity
% alpha = alpha parameter
% L = vector of lambda weights
% sigma = vector of volatilities

% Number of mixturers
N = length(sigma);

% moneyness term
m = log(S0/K) + (r-q)*T;

% Eta quantities when sigma=constant
eta = sigma;

% Sums required for the ATM forward implied vol
term0 = zeros(1,N);
for i=1:N
    term0(i) = L(i) * normcdf(0.5*eta(i)*sqrt(T));
end
Sum0 = sum(term0);

% At-the-money forward implied vol, Equation (4.6) of Brigo and Mercurio
ATMvol = 2/sqrt(T) * norminv((1-alpha)*Sum0 + alpha/2);

% Sums required for the implied vol
term1 = zeros(1,N);
term2 = zeros(1,N);
for i=1:N
    term1(i) = L(i) * normcdf(-0.5*eta(i)*sqrt(T));
    term2(i) = L(i)/eta(i) * exp((ATMvol^2 - eta(i)^2)*T/8);
end
Sum1 = sum(term1);
Sum2 = sum(term2);

% Black Scholes implied vol, Equation (4.5) of Brigo and Mercurio
y = ATMvol ...
    + alpha*sqrt(2*pi/T)*exp(ATMvol^2*T/8)*(Sum1-1/2)*m ...
    + 0.5*( ...
           1/T/(1-alpha)*Sum2 - ...
           1/ATMvol/T + ...
           alpha^2/4*ATMvol*(2*pi)*(Sum1-1/2)^2*(exp(ATMvol^2*T/8))^2 ...
           )*m^2;

