function pdf = BMpdf(y,S,r,q,T,alpha,L,sigma,A0)

% Brigo and Mercurio lognormal mixture risk neutral density
% t = point at which to evaluate the density
% S0 = spot
% r,q = risk free rate, dividend yield
% T = maturity
% alpha = alpha parameter
% L = vector of lambda weights
% sigma = vector of volatilities
% A0 = shift parameter

% Number of lognormal mixtures and drift
N = length(sigma);
mu = r-q;

for i=1:N
    % Shift the t and S
    y0 = y - A0*alpha*exp(mu*T);
    S0 = S - A0*alpha*exp(mu*T);
    % Exponential terms in Equation (3.5) of Brigo and Mercurio
    E(i) = exp(-(log(y0/S0) - mu*T + sigma(i)^2/2)^2/2/sigma(i)^2);
    % The individual pdfs, weighted by L(i) lambda term
    p(i) = L(i)/y0/sigma(i)/sqrt(2*pi)*E(i);
    clear y0 S0
end

% Equation (3.5) of Brigo and Mercurio
pdf = sum(p);    

