% Reproduction of Figures in Brigo and Mercurio (2002)
% "Lognormal-Mixture Dynamics and Calibration to Market Volatility Smiles"
% International Journal of Theoretical and Applied Finance
% Vol 5, No. 4 (2002), pp. 427-446.

clc; clear;


%% Figure 1
S0 = 100;
r = 0.035;
q = 0;
T = 1;
F = S0*exp((r-q)*T);
K = [80:120];
sigma = [0.5 0.1 0.2];
L = [0.2 0.3 0.5];
alpha = 0;
for k=1:length(K)
    IV1(k) = BMIV(K(k),S0,r,q,T,alpha,L,sigma);
end

subplot(2,3,1)
plot(K,IV1,'kx-');
legend('Brigo-Mercurio IV')
line([F F],[.22 .3])
line([K(1) K(end)],[min(IV1) min(IV1)])
title('Figure 1')

%% Figure 2
Y = 40:180;
alpha1 = -0.4;
alpha2 = -0.2;
alpha3 =  0;
alpha4 =  0.2;
A0 = 100;
r = 0.05;
q = 0;
T = 0.5;
sigma = [0.25 0.09 0.04];
L = [0.8 0.1 0.1];

for y=1:length(Y);
    p1(y) = BMpdf(Y(y),S0,r,q,T,alpha1,L,sigma,A0);
    p2(y) = BMpdf(Y(y),S0,r,q,T,alpha2,L,sigma,A0);
    p3(y) = BMpdf(Y(y),S0,r,q,T,alpha3,L,sigma,A0);
    p4(y) = BMpdf(Y(y),S0,r,q,T,alpha4,L,sigma,A0);
end
subplot(2,3,2)
plot(Y,p1,'b--',Y,p2,'g:',Y,p3,'r--',Y,p4,'b-')
legend('alpha = -0.4', 'alpha = -0.2','alpha = 0 ','alpha = 0.2')
title('Figure 2')


%% Figure 3
T = 2;

% Figure 3, top panel
alpha1 =  0;
alpha2 = -0.2;
alpha3 = -0.4;
A0 = 100;
L = [0.6 0.4];
sigma = [0.35000 0.1000];
for k=1:length(K)
    IV3a(k,1) = BMIV(K(k),S0,r,q,T,alpha1,L,sigma);
    IV3a(k,2) = BMIV(K(k),S0,r,q,T,alpha2,L,sigma);
    IV3a(k,3) = BMIV(K(k),S0,r,q,T,alpha3,L,sigma);
end
subplot(2,3,3)
plot(K,IV3a(:,1),'b-',K,IV3a(:,2),'g--',K,IV3a(:,3),'y-')
legend('alpha =  0.0', 'alpha = -0.2', 'alpha = -0.4')
title('Figure 3 top panel')

% Figure 3, bottom
sigma1 = [0.35000 0.1000];
sigma2 = [0.10990 0.3553];
sigma3 = [0.09809 0.2979];

for k=1:length(K)
    IV3b(k,1) = BMIV(K(k),S0,r,q,T,alpha1,L,sigma1);
    IV3b(k,2) = BMIV(K(k),S0,r,q,T,alpha2,L,sigma2);
    IV3b(k,3) = BMIV(K(k),S0,r,q,T,alpha3,L,sigma3);
end
subplot(2,3,4)
plot(K,IV3b(:,1),'b-',K,IV3b(:,2),'g--',K,IV3b(:,3),'y-')
legend('alpha =  0.0', 'alpha = -0.2', 'alpha = -0.4')
title('Figure 3 bottom panel')


%% Figure 4
T = [.25 .5 .75 1 1.25 1.5 1.75 2];
K = 80:120;
r = 0.05;
q = 0;
alpha = -0.5;
L = [0.6 0.4];
sigma = [0.35 0.1];

for t=1:length(T)
    for k=1:length(K)
        IV4(k,t) = BMIV(K(k),S0,r,q,T(t),alpha,L,sigma);
    end
end
subplot(2,3,5)
mesh(K,T,IV4')
zlim([.2 .6])
xlabel('Strike')
ylabel('Maturity')
view(30,30)
title('Figure 4')
     
%% Local volatility in constant sigma case
for t=1:length(T)
    for k=1:length(K);
        LV(k,t) = BMLV(K(k),S0,r,q,T(t),L,sigma,alpha,A0);
    end
end

subplot(2,3,6)
surf(K,T,LV')
hold on
mesh(K,T,IV4')
zlim([.2 .6])
hold off
xlabel('Strike')
ylabel('Maturity')
view(30,30)
title('Implied and Local volatilities')

